@@ -8,14 +8,7 @@ module Excon
       params.delete(:connection)
       params.delete(:stack)
 
-      if params.has_key?(:headers) && params[:headers].has_key?('Authorization')
-        params[:headers] = params[:headers].dup
-        params[:headers]['Authorization'] = REDACTED
-      end
-
-      if params.has_key?(:password)
-        params[:password] = REDACTED
-      end
+      params = Utils.redact(params)
 
       $stderr.puts(name)
       Excon::PrettyPrinter.pp($stderr, params)
